searchData={"items":[{"type":"behaviour","title":"RateLimiterMan","doc":"A rate limiter implementation, based heavily on a blog post by [Alex\nKoutmous](https://akoutmos.com/post/rate-limiting-with-genservers/).\n\nThis package handles logic for limiting the rate at which HTTP requests are sent.","ref":"RateLimiterMan.html"},{"type":"function","title":"RateLimiterMan.get_instance_name/1","doc":"Get the process name for a rate limiter instance by its `config_key`.","ref":"RateLimiterMan.html#get_instance_name/1"},{"type":"function","title":"RateLimiterMan.get_max_requests_per_second/2","doc":"","ref":"RateLimiterMan.html#get_max_requests_per_second/2"},{"type":"function","title":"RateLimiterMan.get_rate_limiter/2","doc":"Get the configured rate limiter for a given `otp_app` and `config_key`.","ref":"RateLimiterMan.html#get_rate_limiter/2"},{"type":"function","title":"Examples - RateLimiterMan.get_rate_limiter/2","doc":"iex> RateLimiterMan.get_rate_limiter(:your_project, YourProject.SomeApi)\n    RateLimiterMan.LeakyBucket","ref":"RateLimiterMan.html#get_rate_limiter/2-examples"},{"type":"function","title":"RateLimiterMan.make_request/4","doc":"Call a function via the rate limiter.\n\nThe `otp_app` must be the atom name of your OTP application, as configured in `mix.exs` (e.g.\n`:your_project`).\n\nThe `config_key` must be the namespace in your project's config where the rate limiter\ninstance's config exists. This value must match the value that was used to create the rate\nlimiter instance in your application's supervision tree.\n\nThe `request_handler` is a tuple that is passed to `Kernel.apply/3`, which represents the\nfunction to be called by the rate limiter.\n\nThe response may be handled in the following ways:\n  - Send the response to a process (e.g. the process that called the rate limiter)\n  - Handle the response as an async task\n  - Do nothing with the response","ref":"RateLimiterMan.html#make_request/4"},{"type":"function","title":"Options - RateLimiterMan.make_request/4","doc":"","ref":"RateLimiterMan.html#make_request/4-options"},{"type":"function","title":"Generic options - RateLimiterMan.make_request/4","doc":"- `:logger_level` - Determines what type of Logger statement to generate when a\nrequest is pushed to or popped from the queue. If this option is not given, the config value\nfor `:rate_limiter_logger_level` in the `config_key` will be used (default: `nil`)\n  - Examples: `nil` (Logging disabled), `:debug`, `:info`\n\n> #### Warning {: .warning}\n>\n> The log statements may contain sensitive data (e.g. API keys). There is currently no way of\n> modifying the contents of the Logger statement.","ref":"RateLimiterMan.html#make_request/4-generic-options"},{"type":"function","title":"Send the response to a process - RateLimiterMan.make_request/4","doc":"To receive a response from the rate limiter, you must pass in the following `opts`:\n\n- `:send_response_to_pid` - The process that will receive the response (default: `nil`)\n  - Examples: `self()`, `pid(0, 1, 2)`\n\n- `:request_id` - A unique identifier for the request, such as a random number or an\n`x-request-id` header) (default: `nil`)\n  - Examples: `\"abc\"`, `123`","ref":"RateLimiterMan.html#make_request/4-send-the-response-to-a-process"},{"type":"function","title":"Handle the response as an async task - RateLimiterMan.make_request/4","doc":"To handle the response by calling a function as an async task, you must pass in the following\n`opts`:\n\n- `:response_handler` - A 2-item tuple that contains the name of the module, and an atom name of\nthe 1-arity function that will handle the response.\n  - Example: `{YourProject.SomeApi, :handle_response}`","ref":"RateLimiterMan.html#make_request/4-handle-the-response-as-an-async-task"},{"type":"function","title":"Do nothing - RateLimiterMan.make_request/4","doc":"To do nothing with the response, just omit any of the options used in the previous handler\nstrategies.","ref":"RateLimiterMan.html#make_request/4-do-nothing"},{"type":"function","title":"Examples - RateLimiterMan.make_request/4","doc":"Make a request with the rate limiter:\n\n    iex> RateLimiterMan.make_request(\n    ...>   _otp_app = :your_project,\n    ...>   _config_key = YourProject.RateLimiter,\n    ...>   _request_handler = {IO, :puts, [\"Hello world!\"]}\n    ...> )\n    :ok","ref":"RateLimiterMan.html#make_request/4-examples"},{"type":"function","title":"Handle the response as an async task - RateLimiterMan.make_request/4","doc":"To handle the response as an async task, you must pass in the following `opts`:\n\n- `:response_handler` - A 2-item tuple that contains the name of the module, and the atom name\nof the 1-arity function that will handle the response.\n  - Example: `{YourProject.SomeApi, :your_response_handler}`","ref":"RateLimiterMan.html#make_request/4-handle-the-response-as-an-async-task"},{"type":"function","title":"Send the response to a process - RateLimiterMan.make_request/4","doc":"Make a request using the rate limiter, and have the rate limiter send the response back to the\ncaller via message passing:\n\nGenerate a unique request ID:\n\n    iex> request_id = System.unique_integer()\n\nMake the request:\n\n    iex> RateLimiterMan.make_request(\n    ...>   _otp_app = :your_project,\n    ...>   _config_key = YourProject.RateLimiter,\n    ...>   _request_handler = {Enum, :join, [[\"Hello\", \"world!\"], \" \"]},\n    ...>   send_response_to_pid: self(),\n    ...>   request_id: request_id\n    ...> )\n    :ok\n\nGet the response back from the rate limiter:\n\n    iex> response = RateLimiterMan.receive_response(request_id)\n    \"Hello world!\"","ref":"RateLimiterMan.html#make_request/4-send-the-response-to-a-process"},{"type":"callback","title":"RateLimiterMan.make_request/4","doc":"","ref":"RateLimiterMan.html#c:make_request/4"},{"type":"function","title":"RateLimiterMan.new_rate_limiter/2","doc":"Add a rate limiter instance to your application's supervision tree.\n\nFor more information on adding a rate limiter to your application, see the [README](README.md).\n\n> #### Tip {: .tip}\n>\n> The TaskSupervisor must be added to your supervision tree before adding any rate limiters.\n> For more information, see `new_task_supervisor/0`.\n\n> #### Tip {: .tip}\n>\n> To temporarily disable a rate limiter when starting your application, change the config for\n> the `:rate_limiter_algorithm` to `RateLimiterMan.None`.","ref":"RateLimiterMan.html#new_rate_limiter/2"},{"type":"function","title":"Examples - RateLimiterMan.new_rate_limiter/2","doc":"`lib/your_project/application.ex`\n```elixir\ndefmodule YourProject.Application do\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children =\n      [\n        # Add the task supervisor before adding any rate limiters\n        RateLimiterMan.new_task_supervisor(),\n        RateLimiterMan.new_rate_limiter(:your_project, YourProject.SomeApi),\n        RateLimiterMan.new_rate_limiter(:your_project, YourProject.SomeOtherApi)\n      ]\n\n    opts = [strategy: :one_for_one, name: YourProject.Supervisor]\n\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"RateLimiterMan.html#new_rate_limiter/2-examples"},{"type":"function","title":"RateLimiterMan.new_task_supervisor/0","doc":"Add a TaskSupervisor to your application's supervision tree.\n\n> #### Tip {: .tip}\n>\n> The TaskSupervisor must be added to your supervision tree before adding any rate limiters.","ref":"RateLimiterMan.html#new_task_supervisor/0"},{"type":"function","title":"Examples - RateLimiterMan.new_task_supervisor/0","doc":"`lib/your_project/application.ex`\n```elixir\ndefmodule YourProject.Application do\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children =\n      [\n        RateLimiterMan.new_task_supervisor()\n      ]\n\n    opts = [strategy: :one_for_one, name: YourProject.Supervisor]\n\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"RateLimiterMan.html#new_task_supervisor/0-examples"},{"type":"function","title":"RateLimiterMan.receive_response/2","doc":"Receive a response from a rate limiter.","ref":"RateLimiterMan.html#receive_response/2"},{"type":"module","title":"RateLimiterMan.LeakyBucket","doc":"A leaky-bucket rate limiter that processes requests at a fixed rate (e.g. 1 request per second).","ref":"RateLimiterMan.LeakyBucket.html"},{"type":"function","title":"RateLimiterMan.LeakyBucket.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"RateLimiterMan.LeakyBucket.html#child_spec/1"},{"type":"function","title":"RateLimiterMan.LeakyBucket.start_link/1","doc":"","ref":"RateLimiterMan.LeakyBucket.html#start_link/1"},{"type":"module","title":"RateLimiterMan.None","doc":"This module acts as a no-op that bypasses any rate limiting.\n\n> #### Warning {: .warning}\n>\n> This algorithm is only intended for temporary usage, and will emit a `Logger` warning on each\n> request. If you want to permanently disable a rate limiter, it should be removed from your\n> project's supervision tree.","ref":"RateLimiterMan.None.html"},{"type":"function","title":"RateLimiterMan.None.child_spec/1","doc":"","ref":"RateLimiterMan.None.html#child_spec/1"},{"type":"function","title":"RateLimiterMan.None.start_link/1","doc":"","ref":"RateLimiterMan.None.html#start_link/1"},{"type":"extras","title":"RateLimiterMan","doc":"# RateLimiterMan\n\nA simple rate limiter implementation, adapted from [a blog post by Alex Koutmous](https://akoutmos.com/post/rate-limiting-with-genservers/).\n\n> #### Warning {: .warning}\n>\n> This is probably not the most performant rate limiter out there, but it solves the need for which it was created: adding rate limiting to an application that receives responses from multiple third-party APIs, each of which has its own rate limiter instance/config.\n\nThis package supports multiple rate limiter instances in your application. Just follow the instructions, using a different config key for each rate limiter you want to add.","ref":"readme.html"},{"type":"extras","title":"Supported rate limiter algorithms - RateLimiterMan","doc":"Currently, the only limiter algorithms implemented by this package are `RateLimiterMan.LeakyBucket`, and `RateLimiterMan.None` (a no-op used to temporarily bypass any configured rate limits).","ref":"readme.html#supported-rate-limiter-algorithms"},{"type":"extras","title":"Getting started - RateLimiterMan","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Install the package - RateLimiterMan","doc":"Add this package to your list of dependencies in `mix.exs`, then run `mix deps.get`:\n\n```elixir\n{:rate_limiter_man, \"0.3.0\"}\n```","ref":"readme.html#install-the-package"},{"type":"extras","title":"Configure your application - RateLimiterMan","doc":"Add the config for your rate limiter instance:\n\n`config/config.exs`\n```elixir\nimport Config\n\nconfig :your_project, YourProject.SomeApi,\n  # Required items\n  rate_limiter_algorithm: RateLimiterMan.LeakyBucket,\n  rate_limiter_max_requests_per_second: 1\n  # Optional items","ref":"readme.html#configure-your-application"},{"type":"extras","title":"rate_limiter_logger_level: :debug # Enable logging when the rate limiter handles a request - RateLimiterMan","doc":"```\n\n> #### Tip {: .tip}\n>\n> Using the example above, the `otp_app` is `:your_project`, and the `config_key` is `YourProject.SomeApi`. These values are used to identify each rate limiter instance by the various functions in this package.\n\n> #### Warning {: .warning}\n>\n> The log statements may contain sensitive data (e.g. API keys). There is currently no way of\n> modifying the contents of the Logger statement.\n\n> #### Tip {: .tip}\n>\n> To temporarily disable a rate limiter when starting your application, change the config for\n> the `:rate_limiter_algorithm` to `RateLimiterMan.None`.","ref":"readme.html#rate_limiter_logger_level-debug-enable-logging-when-the-rate-limiter-handles-a-request"},{"type":"extras","title":"Add the rate limiter to your application's supervision tree - RateLimiterMan","doc":"`lib/your_project/application.ex`\n```elixir\ndefmodule YourProject.Application do\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children =\n      [\n        # Add the task supervisor before adding any rate limiters. The task supervisor should only\n        # be declared once\n        RateLimiterMan.new_task_supervisor(),\n        # Add the desired rate limiter(s). The OTP app name and config key must match the app name\n        # and key used in your config file\n        RateLimiterMan.new_rate_limiter(:your_project, YourProject.SomeApi),\n        # RateLimiterMan.new_rate_limiter(:your_project, YourProject.SomeOtherApi),\n        # RateLimiterMan.new_rate_limiter(:your_project, YourProject.YetAnotherApi)\n      ]\n\n    opts = [strategy: :one_for_one, name: YourProject.Supervisor]\n\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nNow, the rate limiter process will start working when you start your application: `iex -S mix`","ref":"readme.html#add-the-rate-limiter-to-your-application-s-supervision-tree"},{"type":"extras","title":"Example: Use the rate limiter when making a request - RateLimiterMan","doc":"Any function can be passed to the rate limiter. For simplicity, this example will not make any HTTP requests.\n\n`lib/your_project/some_api.ex`\n```\ndefmodule YourProject.SomeApi do\n  def hello_world do\n    request_id = System.unique_integer()\n\n    # Make the request\n    RateLimiterMan.make_request(\n      _otp_app = :your_project,\n      _config_key = YourProject.SomeApi,\n      _request_handler = {Enum, :join, [[\"Hello\", \"world!\"], \" \"]},\n      send_response_to_pid: self(),\n      request_id: request_id\n    )\n\n    # Receive the response\n    response = RateLimiterMan.receive_response(request_id, _timeout = :timer.seconds(5))\n\n    IO.puts(response)\n\n    response\n  end\nend\n```\n\nLet's try it out. Start an interactive shell with `iex -S mix`:\n\n```\niex> for _ <- 1..3, do: YourProject.SomeApi.hello_world()\nHello world!\nHello world!\nHello world!\n[\"Hello world!\", \"Hello world!\", \"Hello world!\"]\n```\n\nIf you set your rate limiter with the config in the instructions, you should see the phrase \"Hello world!\" printed to the terminal 3 times, and the rate limiter ensured that only one \"request\" was handled every second.\n\nIf everything worked, you can now adapt the rate limiter for use in your application.","ref":"readme.html#example-use-the-rate-limiter-when-making-a-request"},{"type":"extras","title":"More information - RateLimiterMan","doc":"Project documentation: https://hexdocs.pm/rate_limiter_man\n\nHex package: https://hex.pm/rate_limiter_man","ref":"readme.html#more-information"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}